@@ -49,7 +49,6 @@
 using namespace Nepomuk2::Vocabulary;
 using namespace Soprano::Vocabulary;
 
-
 Nepomuk2::Resource::Resource()
 {
     QMutexLocker lock( &ResourceManager::instance()->d->mutex );
@@ -450,9 +449,7 @@ void Nepomuk2::Resource::setIdentifiers( const QStringList& value )
 
 void Nepomuk2::Resource::addIdentifier( const QString& value )
 {
-    Variant v = property( Soprano::Vocabulary::NAO::identifier() );
-    v.append( value );
-    setProperty( Soprano::Vocabulary::NAO::identifier(), v );
+    addProperty( Soprano::Vocabulary::NAO::identifier(), Variant( value ) );
 }
 
 
@@ -490,9 +487,7 @@ void Nepomuk2::Resource::addTag( const Nepomuk2::Tag& value )
     // the data and has the advantage that we can mix setProperty calls
     // with the special Resource subclass methods.
     // More importantly Resource loads the data as Resource objects anyway.
-    Variant v = property( Soprano::Vocabulary::NAO::hasTag() );
-    v.append( Resource( value ) );
-    setProperty( Soprano::Vocabulary::NAO::hasTag(), v );
+    addProperty( Soprano::Vocabulary::NAO::hasTag(), Resource(value) );
 }
 
 
@@ -503,7 +498,7 @@ QList<Nepomuk2::Resource> Nepomuk2::Resource::isRelateds() const
     // the data and has the advantage that we can mix setProperty calls
     // with the special Resource subclass methods.
     // More importantly Resource loads the data as Resource objects anyway.
-    return convertResourceList<Resource>( property( Soprano::Vocabulary::NAO::isRelated() ).toResourceList() );
+    return property( Soprano::Vocabulary::NAO::isRelated() ).toResourceList();
 }
 
 
@@ -520,9 +515,7 @@ void Nepomuk2::Resource::addIsRelated( const Nepomuk2::Resource& value )
     // the data and has the advantage that we can mix setProperty calls
     // with the special Resource subclass methods.
     // More importantly Resource loads the data as Resource objects anyway.
-    Variant v = property( Soprano::Vocabulary::NAO::isRelated() );
-    v.append( Resource( value ) );
-    setProperty( Soprano::Vocabulary::NAO::isRelated(), v );
+    addProperty( Soprano::Vocabulary::NAO::isRelated(), Resource(value) );
 }
 
 
@@ -549,6 +542,59 @@ void Nepomuk2::Resource::setRating( const quint32& value )
     setProperty( Soprano::Vocabulary::NAO::numericRating(), Variant( value ) );
 }
 
+QStringList Nepomuk2::Resource::symbols() const
+{
+    QList<Resource> symbolResources = property( Soprano::Vocabulary::NAO::hasSymbol() ).toResourceList();
+
+    QStringList symbolStrings;
+    foreach(const Resource& symbolRes, symbolResources ) {
+        symbolStrings << symbolRes.label();
+    }
+
+    return symbolStrings;
+}
+
+namespace {
+    QUrl uriForSymbolName(const QString& symbolName) {
+        // Check if it exists
+        // We aren't using Soprano::Node::literalToN3 cause prefLabel has a range of a literal not
+        // of a string
+        QString query = QString::fromLatin1("select ?r where { ?r a %1 . ?r %2 \"%3\" . } LIMIT 1")
+        .arg( Soprano::Node::resourceToN3(Soprano::Vocabulary::NAO::FreeDesktopIcon()),
+              Soprano::Node::resourceToN3(Soprano::Vocabulary::NAO::iconName()),
+              symbolName );
+
+        Soprano::Model* model = Nepomuk2::ResourceManager::instance()->mainModel();
+        Soprano::QueryResultIterator it = model->executeQuery( query, Soprano::Query::QueryLanguageSparql );
+        if( it.next() ) {
+            return it["r"].uri();
+        }
+        else {
+            Nepomuk2::Resource res(QUrl(), Soprano::Vocabulary::NAO::FreeDesktopIcon());
+            res.setProperty( NAO::iconName(), symbolName );
+
+            return res.uri();
+        }
+    }
+}
+
+void Nepomuk2::Resource::setSymbols( const QStringList& value )
+{
+    QList<QUrl> symbolList;
+    foreach( const QString& symbolName, value ) {
+        symbolList << uriForSymbolName(symbolName);
+    }
+
+    setProperty( Soprano::Vocabulary::NAO::hasSymbol(), Variant(symbolList) );
+}
+
+
+void Nepomuk2::Resource::addSymbol( const QString& value )
+{
+    addProperty( Soprano::Vocabulary::NAO::hasSymbol(), uriForSymbolName(value) );
+}
+
+
 QList<Nepomuk2::Resource> Nepomuk2::Resource::isRelatedOf() const
 {
     Soprano::Model* model = ResourceManager::instance()->mainModel();
@@ -623,8 +669,8 @@ void Nepomuk2::Resource::determineFinalResourceData() const
     // using the old ResourceData to avoid the overhead of calling
     // determineUri over and over
     if( newData != oldData ) {
-        Q_FOREACH( Resource* res, m_data->m_resources) {
-            res->m_data = newData;
+        Q_FOREACH( Resource* res, m_data->resources() ) {
+            res->m_data = newData; // one of these resources is "this", so this updates our m_data.
             oldData->deref( res );
             newData->ref( res );
         }
