@@ -18,8 +18,8 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef _NEPOMUK_RESOURCE_H_
-#define _NEPOMUK_RESOURCE_H_
+#ifndef _NEPOMUK2_RESOURCE_H_
+#define _NEPOMUK2_RESOURCE_H_
 
 #include <QtCore/QHash>
 #include <QtCore/QStringList>
@@ -435,6 +435,30 @@ namespace Nepomuk2 {
          */
         void setRating( const quint32& value );
 
+        /**
+         * Get property 'Symbol'. Each resource can have a symbol assigned.
+         * For now this is a simple string which can either be the path to
+         * an actual pixmap file or just the name of an icon as defined by
+         * the freedesktop.org standard.
+         */
+        QStringList symbols() const;
+
+        /**
+         * Set property 'Symbol'. Each resource can have a symbol assigned.
+         * For now this is a simple string which can either be the path to
+         * an actual pixmap file or just the name of an icon as defined by
+         * the freedesktop.org standard.
+         */
+        void setSymbols( const QStringList& value );
+
+        /**
+         * Add a value to property 'Symbol'. Each resource can have a symbol
+         * assigned. For now this is a simple string which can either be
+         * the path to an actual pixmap file or just the name of an icon as
+         * defined by the freedesktop.org standard.
+         */
+        void addSymbol( const QString& value );
+
         /**
          * Get all resources that have this resource set as property 'isRelated'.
          * \sa ResourceManager::allResourcesWithProperty
