@@ -32,6 +32,7 @@
 #include "datamanagement.h"
 #include "createresourcejob.h"
 #include "resourcewatcher.h"
+#include "dbustypes.h"
 
 #include <Soprano/Statement>
 #include <Soprano/StatementIterator>
@@ -52,6 +53,9 @@
 #include <kurl.h>
 #include <kcomponentdata.h>
 
+#include <QtDBus/QDBusConnection>
+#include <QtDBus/QDBusMessage>
+
 using namespace Soprano;
 
 #define MAINMODEL (m_rm->m_manager->mainModel())
@@ -162,8 +166,10 @@ void Nepomuk2::ResourceData::resetAll( bool isDelete )
     // This is required cause otherwise the Resource::fromResourceUri creates a new
     // resource which is correctly identified to the ResourceData (this), and it is
     // then deleted, which calls resetAll and this cycle continues.
-    m_rm->m_identifierKickOff.remove( m_cache[NAO::identifier()].toString() );
-    m_rm->m_urlKickOff.remove( m_cache[NIE::url()].toUrl() );
+    const QString nao = m_cache[NAO::identifier()].toString();
+    m_rm->m_identifierKickOff.remove( nao );
+    const QUrl nieUrl = m_cache[NIE::url()].toUrl();
+    m_rm->m_urlKickOff.remove( nieUrl );
 
     if( !m_uri.isEmpty() ) {
         m_rm->m_initializedData.remove( m_uri );
@@ -268,14 +274,26 @@ bool Nepomuk2::ResourceData::store()
         }
         types << m_type;
 
-        Nepomuk2::CreateResourceJob* job = Nepomuk2::createResource(types, QString(), QString());
-        if( !job->exec() ) {
+        QDBusMessage msg = QDBusMessage::createMethodCall( QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("/datamanagement"),
+                                                           QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("createResource") );
+
+        msg.setArguments( QVariantList()
+                          << DBus::convertUriList(types)
+                          << QString() << QString()
+                          << KGlobal::mainComponent().componentName() );
+
+        QDBusConnection bus = QDBusConnection::sessionBus();
+        QDBusMessage reply = bus.call( msg );
+        if( reply.type() == QDBusMessage::ErrorMessage ) {
             //TODO: Set the error somehow
-            kWarning() << job->errorString();
+            kWarning() << reply.errorMessage();
             return false;
         }
         else {
-            m_uri = job->resourceUri();
+            m_uri = reply.arguments().at(0).toUrl();
+
             QList<Soprano::Node> nodes = MAINMODEL->listStatements( m_uri, RDF::type(), QUrl() )
                                          .iterateObjects().allNodes();
             QList<QUrl> types;
@@ -336,37 +354,37 @@ bool Nepomuk2::ResourceData::load()
     QMutexLocker lock(&m_modificationMutex);
 
     if ( m_cacheDirty ) {
+
+        if ( !m_uri.isValid() )
+            return false;
+
+        const QString oldNaoIdentifier = m_cache[NAO::identifier()].toString();
+        const QUrl oldNieUrl = m_cache[NIE::url()].toUrl();
+
         m_cache.clear();
         addToWatcher();
 
-        if ( m_uri.isValid() ) {
-            //
-            // We exclude properties that are part of the inference graph
-            // It would only pollute the user interface
-            //
-            Soprano::QueryResultIterator it = MAINMODEL->executeQuery(QString("select distinct ?p ?o where { "
-                                                                              "%1 ?p ?o . }").arg(Soprano::Node::resourceToN3(m_uri)),
-                                                                      Soprano::Query::QueryLanguageSparqlNoInference);
-            while ( it.next() ) {
-                QUrl p = it["p"].uri();
-                Soprano::Node o = it["o"];
-                Nepomuk2::Variant var = Variant::fromNode( o );
-                //FIXME: Is this updating of the kickoff lists required?
-                // (note: remove 'rmlock'if you remove this)
-                updateKickOffLists( p, var );
-                m_cache[p].append( var );
-            }
+        //
+        // We exclude properties that are part of the inference graph
+        // It would only pollute the user interface
+        //
+        Soprano::QueryResultIterator it = MAINMODEL->executeQuery(QString("select distinct ?p ?o where { "
+                                                                          "%1 ?p ?o . }").arg(Soprano::Node::resourceToN3(m_uri)),
+                                                                  Soprano::Query::QueryLanguageSparqlNoInference);
+        while ( it.next() ) {
+            QUrl p = it["p"].uri();
+            m_cache[p].append( Variant::fromNode( it["o"] ) );
+        }
 
-            m_cacheDirty = false;
-            return true;
-        }
-        else {
-            return false;
-        }
-    }
-    else {
-        return true;
+        const QString newNaoIdentifier = m_cache[NAO::identifier()].toString();
+        const QUrl newNieUrl = m_cache[NIE::url()].toUrl();
+        updateIdentifierLists( oldNaoIdentifier, newNaoIdentifier );
+        updateUrlLists( oldNieUrl, newNieUrl );
+
+        m_cacheDirty = false;
     }
+
+    return true;
 }
 
 
@@ -381,24 +399,34 @@ void Nepomuk2::ResourceData::setProperty( const QUrl& uri, const Nepomuk2::Varia
 
         // update the store
         QVariantList varList;
-        foreach( const Variant& var, value.toVariantList() ) {
-            // make sure resource values are identified and in the store
+        foreach( const Nepomuk2::Variant var, value.toVariantList() ) {
+            // make sure resource values are in the store
             if( var.simpleType() == qMetaTypeId<Resource>() ) {
-                Resource res = var.toResource();
-                res.determineFinalResourceData();
-                res.m_data->store();
-
-                varList << res.uri();
+                var.toResource().m_data->store();
+                varList << var.toUrl();
             }
             else {
                 varList << var.variant();
             }
         }
 
-        KJob* job = Nepomuk2::setProperty(QList<QUrl>() << m_uri, uri, varList);
-        if( !job->exec() ) {
+        // update the store
+        QDBusMessage msg = QDBusMessage::createMethodCall( QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("/datamanagement"),
+                                                           QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("setProperty") );
+
+        msg.setArguments( QVariantList()
+                          << DBus::convertUriList(QList<QUrl>() << m_uri)
+                          << DBus::convertUri(uri)
+                          << QVariant(DBus::normalizeVariantList(varList))
+                          << KGlobal::mainComponent().componentName() );
+
+        QDBusConnection bus = QDBusConnection::sessionBus();
+        QDBusMessage reply = bus.call( msg );
+        if( reply.type() == QDBusMessage::ErrorMessage ) {
             //TODO: Set the error somehow
-            kWarning() << job->errorString();
+            kWarning() << reply.errorMessage();
             return;
         }
 
@@ -429,21 +457,29 @@ void Nepomuk2::ResourceData::addProperty( const QUrl& uri, const Nepomuk2::Varia
         foreach( const Nepomuk2::Variant var, value.toVariantList() ) {
             // make sure resource values are in the store
             if( var.simpleType() == qMetaTypeId<Resource>() ) {
-                Resource res = var.toResource();
-                res.determineFinalResourceData();
-                res.m_data->store();
-
-                varList << res.uri();
+                var.toResource().m_data->store();
+                varList << var.toUrl();
             }
             else {
                 varList << var.variant();
             }
         }
 
-        KJob* job = Nepomuk2::addProperty(QList<QUrl>() << m_uri, uri, varList);
-        if( !job->exec() ) {
+        QDBusMessage msg = QDBusMessage::createMethodCall( QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("/datamanagement"),
+                                                           QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("addProperty") );
+        msg.setArguments( QVariantList()
+                          << DBus::convertUriList(QList<QUrl>() << m_uri)
+                          << DBus::convertUri(uri)
+                          << QVariant(DBus::normalizeVariantList(varList))
+                          << KGlobal::mainComponent().componentName() );
+
+        QDBusConnection bus = QDBusConnection::sessionBus();
+        QDBusMessage reply = bus.call( msg );
+        if( reply.type() == QDBusMessage::ErrorMessage ) {
             //TODO: Set the error somehow
-            kWarning() << job->errorString();
+            kWarning() << reply.errorMessage();
             return;
         }
 
@@ -465,10 +501,20 @@ void Nepomuk2::ResourceData::removeProperty( const QUrl& uri )
     QMutexLocker lock(&m_modificationMutex);
 
     if( !m_uri.isEmpty() ) {
-        KJob* job = Nepomuk2::removeProperties(QList<QUrl>() << m_uri, QList<QUrl>() << uri);
-        if( !job->exec() ) {
+        QDBusMessage msg = QDBusMessage::createMethodCall( QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("/datamanagement"),
+                                                           QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("removeProperties") );
+        msg.setArguments( QVariantList()
+                          << DBus::convertUri(m_uri)
+                          << DBus::convertUri(uri)
+                          << KGlobal::mainComponent().componentName() );
+
+        QDBusConnection bus = QDBusConnection::sessionBus();
+        QDBusMessage reply = bus.call( msg );
+        if( reply.type() == QDBusMessage::ErrorMessage ) {
             //TODO: Set the error somehow
-            kWarning() << job->errorString();
+            kWarning() << reply.errorMessage();
             return;
         }
 
@@ -488,10 +534,20 @@ void Nepomuk2::ResourceData::remove( bool recursive )
     QMutexLocker lock(&m_modificationMutex);
 
     if( !m_uri.isEmpty() ) {
-        KJob* job = Nepomuk2::removeResources(QList<QUrl>() << m_uri);
-        if( !job->exec() ) {
+        QDBusMessage msg = QDBusMessage::createMethodCall( QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("/datamanagement"),
+                                                           QLatin1String("org.kde.nepomuk.DataManagement"),
+                                                           QLatin1String("removeResources") );
+        msg.setArguments( QVariantList()
+                          << DBus::convertUriList(QList<QUrl>() << m_uri)
+                          << 0 /* no flags */
+                          << KGlobal::mainComponent().componentName());
+
+        QDBusConnection bus = QDBusConnection::sessionBus();
+        QDBusMessage reply = bus.call( msg );
+        if( reply.type() == QDBusMessage::ErrorMessage ) {
             //TODO: Set the error somehow
-            kWarning() << job->errorString();
+            kWarning() << reply.errorMessage();
             return;
         }
     }
@@ -643,22 +699,22 @@ QDebug operator<<( QDebug dbg, const Nepomuk2::ResourceData& data )
     return data.operator<<( dbg );
 }
 
-void Nepomuk2::ResourceData::updateUrlLists(const QUrl& newUrl)
+void Nepomuk2::ResourceData::updateUrlLists(const QUrl& oldUrl, const QUrl& newUrl)
 {
-    const QUrl oldUrl = m_cache[NIE::url()].toUrl();
-
-    m_rm->m_urlKickOff.remove( oldUrl );
+    if ( !oldUrl.isEmpty() ) {
+        m_rm->m_urlKickOff.remove( oldUrl );
+    }
 
     if( !newUrl.isEmpty() ) {
         m_rm->m_urlKickOff.insert( newUrl, this );
     }
 }
 
-void Nepomuk2::ResourceData::updateIdentifierLists(const QString& newIdentifier)
+void Nepomuk2::ResourceData::updateIdentifierLists(const QString& oldIdentifier, const QString& newIdentifier)
 {
-    const QString oldIdentifier = m_cache[NAO::identifier()].toString();
-
-    m_rm->m_identifierKickOff.remove( oldIdentifier );
+    if ( !oldIdentifier.isEmpty() ) {
+        m_rm->m_identifierKickOff.remove( oldIdentifier );
+    }
 
     if( !newIdentifier.isEmpty() ) {
         m_rm->m_identifierKickOff.insert( newIdentifier, this );
@@ -669,9 +725,9 @@ void Nepomuk2::ResourceData::updateIdentifierLists(const QString& newIdentifier)
 void Nepomuk2::ResourceData::updateKickOffLists(const QUrl& uri, const Nepomuk2::Variant& variant)
 {
     if( uri == NIE::url() )
-        updateUrlLists( variant.toUrl() );
+        updateUrlLists( m_cache[NIE::url()].toUrl(), variant.toUrl() );
     else if( uri == NAO::identifier() )
-        updateIdentifierLists( variant.toString() );
+        updateIdentifierLists( m_cache[NAO::identifier()].toString(), variant.toString() );
 }
 
 void Nepomuk2::ResourceData::propertyRemoved( const Types::Property &prop, const QVariant &value_ )
