@@ -211,7 +211,22 @@ Nepomuk2::ResourceManager::ResourceManager()
     connect( watcher, SIGNAL(serviceUnregistered(QString)),
              this, SLOT(_k_dbusServiceUnregistered(QString)) );
 
-    init();
+    // now let's see if the service is already running and init, if so
+    QDBusMessage result = QDBusConnection::sessionBus().call(
+            QDBusMessage::createMethodCall(
+                    QLatin1String("org.kde.NepomukStorage"),
+                    QLatin1String("/servicecontrol"),
+                    QLatin1String("org.kde.nepomuk.ServiceControl"),
+                    QLatin1String("isInitialized")
+                )
+            );
+    if ( result.type() == QDBusMessage::ReplyMessage ) {
+        if ( result.arguments().count() > 0 ) {
+            const bool initialized = result.arguments()[0].toBool();
+            d->_k_storageServiceInitialized( initialized );
+        }
+    }
+
 }
 
 
@@ -245,6 +260,10 @@ int Nepomuk2::ResourceManager::init()
 {
     QMutexLocker lock( &d->initMutex );
 
+    if( d->overrideModel ) {
+        return true;
+    }
+
     if( !d->mainModel ) {
         d->mainModel = new MainModel( this );
     }
@@ -258,6 +277,9 @@ int Nepomuk2::ResourceManager::init()
 bool Nepomuk2::ResourceManager::initialized() const
 {
     QMutexLocker lock( &d->initMutex );
+    if( d->overrideModel )
+        return true;
+
     return d->mainModel && d->mainModel->isValid();
 }
 
@@ -314,6 +336,9 @@ Soprano::Model* Nepomuk2::ResourceManager::mainModel()
         init();
     }
 
+    if( d->overrideModel )
+        return d->overrideModel;
+
     return d->mainModel;
 }
 
