@@ -21,6 +21,8 @@
 
 #include <QtCore/QStringList>
 #include <QtCore/QDir>
+#include <QWriteLocker>
+#include <QReadLocker>
 
 #include <KDirWatch>
 #include <KStandardDirs>
@@ -128,17 +130,6 @@ QStringList Nepomuk2::FileIndexerConfig::excludeFilters() const
     return QStringList::fromSet(filters);
 }
 
-QStringList Nepomuk2::FileIndexerConfig::excludePlugins() const
-{
-    KConfigGroup cfg = m_config.group( "General" );
-    //Read list of strigi plugins not to use
-    //DigestEventPlugin is a SHA1 indexer.
-    QSet<QString> filters = cfg.readEntry( "exclude plugins", QStringList() << "DigestEventAnalyzer" ).toSet();
-
-    // remove duplicates
-    return QStringList::fromSet(filters);
-}
-
 
 bool Nepomuk2::FileIndexerConfig::indexHiddenFilesAndFolders() const
 {
@@ -179,6 +170,10 @@ bool Nepomuk2::FileIndexerConfig::shouldBeIndexed( const QString& path ) const
     }
 }
 
+bool Nepomuk2::FileIndexerConfig::shouldFolderBeWatched( const QString& path ) const
+{
+    return true;
+}
 
 bool Nepomuk2::FileIndexerConfig::shouldFolderBeIndexed( const QString& path ) const
 {
@@ -216,14 +211,14 @@ bool Nepomuk2::FileIndexerConfig::shouldFolderBeIndexed( const QString& path ) c
 bool Nepomuk2::FileIndexerConfig::shouldFileBeIndexed( const QString& fileName ) const
 {
     // check the filters
-    QMutexLocker lock( &m_folderCacheMutex );
+    QWriteLocker lock( &m_folderCacheMutex );
     return !m_excludeFilterRegExpCache.exactMatch( fileName );
 }
 
 
 bool Nepomuk2::FileIndexerConfig::folderInFolderList( const QString& path, QString& folder ) const
 {
-    QMutexLocker lock( &m_folderCacheMutex );
+    QReadLocker lock( &m_folderCacheMutex );
 
     const QString p = KUrl( path ).path( KUrl::RemoveTrailingSlash );
 
@@ -297,7 +292,7 @@ namespace {
 
 void Nepomuk2::FileIndexerConfig::buildFolderCache()
 {
-    QMutexLocker lock( &m_folderCacheMutex );
+    QWriteLocker lock( &m_folderCacheMutex );
 
     QStringList includeFoldersPlain = m_config.group( "General" ).readPathEntry( "folders", QStringList() << QDir::homePath() );
     QStringList excludeFoldersPlain = m_config.group( "General" ).readPathEntry( "exclude folders", QStringList() );
@@ -311,7 +306,7 @@ void Nepomuk2::FileIndexerConfig::buildFolderCache()
 
 void Nepomuk2::FileIndexerConfig::buildExcludeFilterRegExpCache()
 {
-    QMutexLocker lock( &m_folderCacheMutex );
+    QWriteLocker lock( &m_folderCacheMutex );
     m_excludeFilterRegExpCache.rebuildCacheFromFilterList( excludeFilters() );
 }
 
@@ -343,15 +338,5 @@ bool Nepomuk2::FileIndexerConfig::isDebugModeEnabled() const
     return m_config.group( "General" ).readEntry( "debug mode", false );
 }
 
-QString Nepomuk2::FileIndexerConfig::strigiVersion() const
-{
-    return m_config.group( "General" ).readEntry( "strigiVersion", QString() );
-}
-
-void Nepomuk2::FileIndexerConfig::setStrigiVersion(const QString& version)
-{
-    m_config.group( "General" ).writeEntry( "strigiVersion", version );
-}
-
 
 #include "fileindexerconfig.moc"
