@@ -27,16 +27,31 @@
 #include "simpleresource.h"
 #include "simpleresourcegraph.h"
 
+#include <KJob>
+#include <KUrl>
+
 namespace Nepomuk2 {
-    class SimpleIndexer
+
+    class SimpleIndexingJob : public KJob
     {
+        Q_OBJECT
     public:
-        SimpleIndexer(const QUrl& fileUrl);
-        bool save();
+        SimpleIndexingJob(const QUrl& fileUrl, QObject* parent = 0);
+
+        virtual void start();
+
+        QUrl uri();
+        QString mimeType();
+
+    private slots:
+        void slotJobFinished(KJob* job);
 
-        static QList<QUrl> typesForMimeType(const QString& mimeType);
     private:
-        SimpleResourceGraph m_graph;
+        KUrl m_nieUrl;
+        QUrl m_resUri;
+        QString m_mimeType;
+
+        QList<QUrl> typesForMimeType(const QString& mimeType);
     };
 }
 
