@@ -65,8 +65,7 @@ class KInotify::Private
 {
 public:
     Private( KInotify* parent )
-        : watchHiddenFolders( false ),
-          m_inotifyFd( -1 ),
+        : m_inotifyFd( -1 ),
           m_notifier( 0 ),
           q( parent) {
     }
@@ -88,8 +87,6 @@ public:
     WatchEvents mode;
     WatchFlags flags;
 
-    bool watchHiddenFolders;
-
     int inotify() {
         if ( m_inotifyFd < 0 ) {
             open();
@@ -107,15 +104,21 @@ public:
     }
 
     bool addWatch( const QByteArray& path ) {
-        // we always need the unmount event to maintain our path hash
         WatchEvents newMode = mode;
         WatchFlags newFlags = flags;
 
         if( !q->filterWatch( path, newMode, newFlags ) ) {
             return true;
         }
-        const int mask = newMode|newFlags|EventUnmount;
+        // we always need the unmount event to maintain our path hash
+        const int mask = newMode|newFlags|EventUnmount|FlagExclUnlink;
 
+        return addWatchNoCheck(path, mask);
+    }
+
+    /*This function adds a watch assuming the filtering has already been done,
+     * so it can be used in both addWatch and addWatchRecursive */
+    bool addWatchNoCheck( const QByteArray& path, const int mask ) {
         int wd = inotify_add_watch( inotify(), path.data(), mask );
         if ( wd > 0 ) {
 //            kDebug() << "Successfully added watch for" << path << pathHash.count();
@@ -142,13 +145,63 @@ public:
         inotify_rm_watch( inotify(), wd );
     }
 
-    void _k_addWatches() {
-        // add the next batch of paths
-        for ( int i = 0; i < 100 && !dirIterators.isEmpty(); ++i ) {
+    /*Number of watches to add in one go, before deferring the rest to a future invocation*/
+    #define WATCHES_AT_ONCE 200
+
+    /*Add watches recursively. We don't use the QDir::Subfolders attribute
+     * because we want to avoid walking the directory tree in subfolders we aren't indexing:
+     * they may be large, complicated and mounted over a network.*/
+    bool addWatchesRecursively(const QString& path, int& count){
+        WatchEvents newMode = mode;
+        WatchFlags newFlags = flags;
+        //If we do not want to watch this directory, go home, reporting success.
+        if(!q->filterWatch( path, newMode, newFlags )){
+            return true;
+        }
+
+        // We always need the unmount event to maintain our path hash
+        const int mask = newMode|newFlags|EventUnmount|FlagExclUnlink;
+        //Try to install a watch on this directory.
+        if(!addWatchNoCheck( QFile::encodeName(path) , mask )){
+            return false;
+        }
+        //We installed a watch, so increment our counter.
+        count++;
+        //Next try to add subdirectories, calling self recursively.
+        QDirIterator* it = new QDirIterator( path, QDir::Dirs | QDir::NoDotAndDotDot);
+        //If we have added enough directories already,
+        //queue any subdirectories in this one for later addition
+        //and stop.
+        if(count > WATCHES_AT_ONCE+25){
+            if(it->hasNext()){
+                dirIterators.append(it);
+            }
+            return true;
+        }
+        //Note that because we check count before this, if count=99 here,
+        //we may add all top-level subdirectories to the queue.
+        while(it->hasNext()){
+            if(!addWatchesRecursively(it->next(),count)){
+                return false;
+            }
+        }
+        return true;
+    }
+
+    bool _k_addWatches() {
+        /* add the next batch of paths
+         * i is incrememented inside addWatchesRecursively
+         * Note that we allow i to become a bit larger than WATCHES_AT_ONCE
+         * in addWatchesRecursively, to starting a new tree near the end of a batch.
+         */
+        for ( int i = 0; i < WATCHES_AT_ONCE && !dirIterators.isEmpty(); ) {
             QDirIterator* it = dirIterators.front();
             if( it->hasNext() ) {
                 it->next();
-                addWatch( QFile::encodeName(it->filePath()) );
+                //Try to recursively install watches on this directory.
+                if( !addWatchesRecursively(it->filePath(),i) ){
+                    return false;
+                }
             }
             else {
                 delete dirIterators.dequeue();
@@ -156,10 +209,11 @@ public:
             }
         }
 
-        // asyncroneously add the next batch
+        // asynchronously add the next batch
         if ( !dirIterators.isEmpty() ) {
             QMetaObject::invokeMethod( q, "_k_addWatches", Qt::QueuedConnection );
         }
+	return true;
     }
 
 private:
@@ -235,12 +289,11 @@ bool KInotify::addWatch( const QString& path, WatchEvents mode, WatchFlags flags
 
     d->mode = mode;
     d->flags = flags;
-    d->addWatch( QFile::encodeName(path) );
-    QDirIterator* iter = new QDirIterator( path, QDir::Dirs | QDir::NoDotAndDotDot,
-                                           QDirIterator::Subdirectories );
+    if(! (d->addWatch( QFile::encodeName(path) )))
+        return false;
+    QDirIterator* iter = new QDirIterator( path, QDir::Dirs | QDir::NoDotAndDotDot);
     d->dirIterators.append( iter );
-    d->_k_addWatches();
-    return true;
+    return d->_k_addWatches();
 }
 
 
@@ -295,11 +348,7 @@ void KInotify::slotEvent( int socket )
         }
 
         Q_ASSERT( !path.isEmpty() || event->mask & EventIgnored );
-        // This is present cause a unmount event is sent my inotify after unmounting, by
-        // which time the watches have already been removed.
-        if( path == "/" && event->mask & EventUnmount )
-            break;
-        Q_ASSERT( path != "/" || event->mask & EventIgnored );
+        Q_ASSERT( path != "/" || event->mask & EventIgnored  || event->mask & EventUnmount);
 
         // now signal the event
         if ( event->mask & EventAccess) {
@@ -387,7 +436,11 @@ void KInotify::slotEvent( int socket )
             if ( event->mask & IN_ISDIR ) {
                 d->removeWatch( event->wd );
             }
-            emit unmounted( QFile::decodeName(path) );
+            // This is present because a unmount event is sent by inotify after unmounting, by
+            // which time the watches have already been removed.
+            if( path != "/"){
+                emit unmounted( QFile::decodeName(path) );
+            }
         }
         if ( event->mask & EventQueueOverflow ) {
             // This should not happen since we grab all events as soon as they arrive
@@ -395,7 +448,7 @@ void KInotify::slotEvent( int socket )
 //            emit queueOverflow();
         }
         if ( event->mask & EventIgnored ) {
-            kDebug() << path << "EventIgnored";
+//             kDebug() << path << "EventIgnored";
         }
 
         i += EVENT_STRUCT_SIZE + event->len;
